package com.fjhx.service.logistics.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.constants.logistics.LogisticsConstant;
import com.fjhx.entity.apply.ApplyPurchase;
import com.fjhx.entity.logistics.LogisticsInfo;
import com.fjhx.entity.purchase.Purchase;
import com.fjhx.enums.purchase.PurchaseStatusEnum;
import com.fjhx.mapper.logistics.LogisticsInfoMapper;
import com.fjhx.params.logistics.LogisticsInfoVo;
import com.fjhx.service.logistics.LogisticsDetailsService;
import com.fjhx.service.logistics.LogisticsInfoService;
import com.fjhx.uitl.kd100.KD100Util;
import com.fjhx.utils.FileClientUtil;
import com.fjhx.utils.UserClientUtil;
import com.fjhx.utils.wrapperUtil.IWrapper;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.tool.utils.Func;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 物流信息 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-11-30
 */
@Service
public class LogisticsInfoServiceImpl extends ServiceImpl<LogisticsInfoMapper, LogisticsInfo> implements LogisticsInfoService {

    @Autowired
    private LogisticsDetailsService logisticsDetailsService;

    @Override
    public Page<LogisticsInfo> getPage(Map<String, Object> condition) {

        IWrapper<LogisticsInfo> wrapper = IWrapper.getWrapper(condition);

        return page(condition, wrapper);
    }

    /**
     * 采购入库列表
     *
     * @param condition 查询条件
     * @return
     */
    @Override
    public Page<LogisticsInfo> getPurchaseInStockPage(Map<String, Object> condition) {

        IWrapper<LogisticsInfo> wrapper = IWrapper.getWrapper(condition);
        wrapper.eq("t1", LogisticsInfo::getStatus, LogisticsConstant.Status.STATUS_1)
                .lt("t1", LogisticsInfo::getInStockStatus, LogisticsConstant.InStockStatus.STATUS_30)
                .eq("t1", LogisticsInfo::getBusinessType, 1)
                .ge("t2", Purchase::getStatus, PurchaseStatusEnum.STATUS_30.getKey())
                .le("t2", Purchase::getStatus, PurchaseStatusEnum.STATUS_40.getKey())
                .eq("t1", LogisticsInfo::getInStockStatus)
                .and(Func.isNotEmpty(condition.get("keyword")), o -> o.apply("instr(t1.`code`, '" + condition.get("keyword") + "') > 0").or().apply("instr(t2.`code`, '" + condition.get("keyword") + "') > 0").or().apply("instr(t3.`name`, '" + condition.get("keyword") + "') > 0"))
                .orderByDesc("t1", ApplyPurchase::getId);

        Page<LogisticsInfo> page = baseMapper.getPurchaseInStockPage(createPage(condition), wrapper);
        if (Func.isNotEmpty(page.getRecords())) {
            Map<Long, String> userNameMap = UserClientUtil.getUserNameMap(page.getRecords(), LogisticsInfo::getCreateUser);
            for (LogisticsInfo entity : page.getRecords()) {
                if (Func.isNotEmpty(userNameMap) && Func.isNotEmpty(userNameMap.get(entity.getCreateUser()))) {
                    entity.setPurchaseName(userNameMap.get(entity.getCreateUser()));
                }
            }
        }

        return page;
    }

    @Transactional(rollbackFor = {Exception.class})
    @Override
    public void add(LogisticsInfoVo logisticsInfoVo) {
        if (Func.isAnyBlank(logisticsInfoVo.getLogisticsCompanyCode(), logisticsInfoVo.getCode())) {
            throw new ServiceException("物流信息不能为空");
        }

        //查询快递100的物流信息
        JSONObject result = KD100Util.queryTrack(logisticsInfoVo.getLogisticsCompanyCode(), logisticsInfoVo.getCode());
        Integer state = result.getInteger("state");
        if (state != LogisticsConstant.KD100Status.STATUS_3) {
            //如果不是已签收状态，则开启订阅（物流状态跟踪并推送）
            KD100Util.subscribe(logisticsInfoVo.getLogisticsCompanyCode(), logisticsInfoVo.getCode());
        }
        logisticsInfoVo.setLogisticsStatus(state);
        logisticsInfoVo.setStatus(LogisticsConstant.Status.STATUS_0);
        logisticsInfoVo.setInStockStatus(LogisticsConstant.InStockStatus.STATUS_10);
        save(logisticsInfoVo);

        logisticsInfoVo.getDetails().forEach(o -> {
            o.setLogisticsInfoId(logisticsInfoVo.getId());
            o.setLogisticsInfoCode(logisticsInfoVo.getCode());
            o.setReceiptQuantity(new BigDecimal(BigDecimal.ZERO.intValue()));
        });

        //保存附件
        FileClientUtil.bindingFile(logisticsInfoVo.getId(), logisticsInfoVo.getFileInfos());

        logisticsDetailsService.saveBatch(logisticsInfoVo.getDetails());
    }

    @Override
    public void edit(LogisticsInfoVo logisticsInfoVo) {
        updateById(logisticsInfoVo);
    }

    @Override
    public void delete(LogisticsInfoVo logisticsInfoVo) {
        removeById(logisticsInfoVo.getId());
    }

    /**
     * 根据采购ID查询
     *
     * @param purchaseIds 采购ID集合
     * @return
     */
    @Override
    public List<LogisticsInfo> getByPurchaseIds(List<Long> purchaseIds) {
        if (Func.isEmpty(purchaseIds)) {
            return new ArrayList<>();
        }
        return lambdaQuery().in(LogisticsInfo::getBusinessId, purchaseIds).list();
    }

    /**
     * 根据采购ID查询
     *
     * @param purchaseIds 采购ID集合
     * @return
     */
    @Override
    public Map<Long, List<LogisticsInfo>> getByPurchaseIdsToMap(List<Long> purchaseIds) {
        List<LogisticsInfo> list = getByPurchaseIds(purchaseIds);
        if (Func.isEmpty(list)) {
            return new HashMap<>();
        }
        return list.stream().collect(Collectors.groupingBy(LogisticsInfo::getBusinessId));
    }

    /**
     * 根据物流单号跟新物流状态
     *
     * @param code  物流单号
     * @param state 物流状态
     */
    @Override
    public void updateStatusByCode(String code, Integer state) {
        lambdaUpdate().set(LogisticsInfo::getStatus, state).eq(LogisticsInfo::getCode, code).update();
    }

    /**
     * 根据业务ID查询物流单号
     *
     * @param businessId 业务ID
     * @return
     */
    @Override
    public List<LogisticsInfo> codeSelectByBusinessId(Long businessId) {
        return lambdaQuery()
                .select(LogisticsInfo::getId, LogisticsInfo::getCode)
                .eq(LogisticsInfo::getBusinessType, 1)
                .eq(LogisticsInfo::getBusinessId, businessId)
                .eq(LogisticsInfo::getStatus, LogisticsConstant.Status.STATUS_0)
                .list();
    }
}
